@@ -56,7 +56,7 @@ function server() {
     }, dur * 1000);
   });
 
-  socket.on('message', function(buf, rinfo) {
+  socket.on('message', function() {
     received++;
   });
 
