@@ -11,14 +11,14 @@
 var common = require('../common.js');
 
 var bench = common.createBenchmark(main, {
-  num: [1, 4, 8, 16],
-  size: [1, 64, 256],
+  n: [1, 4, 8, 16],
+  len: [1, 64, 256],
   c: [100]
 });
 
 function main(conf) {
   const http = require('http');
-  var chunk = Buffer.alloc(conf.size, '8');
+  var chunk = Buffer.alloc(conf.len, '8');
 
   var server = http.createServer(function(req, res) {
     function send(left) {
@@ -28,7 +28,7 @@ function main(conf) {
         send(left - 1);
       }, 0);
     }
-    send(conf.num);
+    send(conf.n);
   });
 
   server.listen(common.PORT, function() {
