@@ -1,6 +1,6 @@
+'use strict';
 var common = require('../common.js');
 var path = require('path');
-var v8 = require('v8');
 
 var bench = common.createBenchmark(main, {
   path: [
@@ -18,13 +18,7 @@ var bench = common.createBenchmark(main, {
 function main(conf) {
   var n = +conf.n;
   var p = path.posix;
-  var input = '' + conf.path;
-
-  // Force optimization before starting the benchmark
-  p.dirname(input);
-  v8.setFlagsFromString('--allow_natives_syntax');
-  eval('%OptimizeFunctionOnNextCall(p.dirname)');
-  p.dirname(input);
+  var input = String(conf.path);
 
   bench.start();
   for (var i = 0; i < n; i++) {
