@@ -1,6 +1,6 @@
+'use strict';
 var common = require('../common.js');
 var path = require('path');
-var v8 = require('v8');
 
 var bench = common.createBenchmark(main, {
   paths: [
@@ -18,22 +18,19 @@ var bench = common.createBenchmark(main, {
 function main(conf) {
   var n = +conf.n;
   var p = path.posix;
-  var from = '' + conf.paths;
+  var from = String(conf.paths);
   var to = '';
   var delimIdx = from.indexOf('|');
   if (delimIdx > -1) {
     to = from.slice(delimIdx + 1);
     from = from.slice(0, delimIdx);
   }
-
-  // Force optimization before starting the benchmark
-  p.relative(from, to);
-  v8.setFlagsFromString('--allow_natives_syntax');
-  eval('%OptimizeFunctionOnNextCall(p.relative)');
-  p.relative(from, to);
+  for (var i = 0; i < n; i++) {
+    p.relative(from, to);
+  }
 
   bench.start();
-  for (var i = 0; i < n; i++) {
+  for (i = 0; i < n; i++) {
     p.relative(from, to);
   }
   bench.end(n);
