@@ -26,6 +26,37 @@ namespace compiler {
 class Operator;
 struct JSOperatorGlobalCache;
 
+// Defines the frequency a given Call/Construct site was executed. For some
+// call sites the frequency is not known.
+class CallFrequency final {
+ public:
+  CallFrequency() : value_(std::numeric_limits<float>::quiet_NaN()) {}
+  explicit CallFrequency(float value) : value_(value) {
+    DCHECK(!std::isnan(value));
+  }
+
+  bool IsKnown() const { return !IsUnknown(); }
+  bool IsUnknown() const { return std::isnan(value_); }
+  float value() const {
+    DCHECK(IsKnown());
+    return value_;
+  }
+
+  bool operator==(CallFrequency const& that) const {
+    return bit_cast<uint32_t>(this->value_) == bit_cast<uint32_t>(that.value_);
+  }
+  bool operator!=(CallFrequency const& that) const { return !(*this == that); }
+
+  friend size_t hash_value(CallFrequency f) {
+    return bit_cast<uint32_t>(f.value_);
+  }
+
+ private:
+  float value_;
+};
+
+std::ostream& operator<<(std::ostream&, CallFrequency);
+
 // Defines a pair of {FeedbackVector} and {FeedbackSlot}, which
 // is used to access the type feedback for a certain {Node}.
 class V8_EXPORT_PRIVATE VectorSlotPair {
@@ -59,22 +90,56 @@ ConvertReceiverMode ConvertReceiverModeOf(Operator const* op);
 // The ToBooleanHints are used as parameter by JSToBoolean operators.
 ToBooleanHints ToBooleanHintsOf(Operator const* op);
 
+// Defines the flags for a JavaScript call forwarding parameters. This
+// is used as parameter by JSConstructForwardVarargs operators.
+class ConstructForwardVarargsParameters final {
+ public:
+  ConstructForwardVarargsParameters(size_t arity, uint32_t start_index)
+      : bit_field_(ArityField::encode(arity) |
+                   StartIndexField::encode(start_index)) {}
+
+  size_t arity() const { return ArityField::decode(bit_field_); }
+  uint32_t start_index() const { return StartIndexField::decode(bit_field_); }
+
+  bool operator==(ConstructForwardVarargsParameters const& that) const {
+    return this->bit_field_ == that.bit_field_;
+  }
+  bool operator!=(ConstructForwardVarargsParameters const& that) const {
+    return !(*this == that);
+  }
+
+ private:
+  friend size_t hash_value(ConstructForwardVarargsParameters const& p) {
+    return p.bit_field_;
+  }
+
+  typedef BitField<size_t, 0, 16> ArityField;
+  typedef BitField<uint32_t, 16, 16> StartIndexField;
+
+  uint32_t const bit_field_;
+};
+
+std::ostream& operator<<(std::ostream&,
+                         ConstructForwardVarargsParameters const&);
+
+ConstructForwardVarargsParameters const& ConstructForwardVarargsParametersOf(
+    Operator const*) WARN_UNUSED_RESULT;
 
 // Defines the arity and the feedback for a JavaScript constructor call. This is
 // used as a parameter by JSConstruct operators.
 class ConstructParameters final {
  public:
-  ConstructParameters(uint32_t arity, float frequency,
+  ConstructParameters(uint32_t arity, CallFrequency frequency,
                       VectorSlotPair const& feedback)
       : arity_(arity), frequency_(frequency), feedback_(feedback) {}
 
   uint32_t arity() const { return arity_; }
-  float frequency() const { return frequency_; }
+  CallFrequency frequency() const { return frequency_; }
   VectorSlotPair const& feedback() const { return feedback_; }
 
  private:
   uint32_t const arity_;
-  float const frequency_;
+  CallFrequency const frequency_;
   VectorSlotPair const feedback_;
 };
 
@@ -87,12 +152,12 @@ std::ostream& operator<<(std::ostream&, ConstructParameters const&);
 
 ConstructParameters const& ConstructParametersOf(Operator const*);
 
-// Defines the arity for a JavaScript constructor call with a spread as the last
-// parameters. This is used as a parameter by JSConstructWithSpread
-// operators.
-class ConstructWithSpreadParameters final {
+// Defines the arity for JavaScript calls with a spread as the last
+// parameter. This is used as a parameter by JSConstructWithSpread and
+// JSCallWithSpread operators.
+class SpreadWithArityParameter final {
  public:
-  explicit ConstructWithSpreadParameters(uint32_t arity) : arity_(arity) {}
+  explicit SpreadWithArityParameter(uint32_t arity) : arity_(arity) {}
 
   uint32_t arity() const { return arity_; }
 
@@ -100,27 +165,28 @@ class ConstructWithSpreadParameters final {
   uint32_t const arity_;
 };
 
-bool operator==(ConstructWithSpreadParameters const&,
-                ConstructWithSpreadParameters const&);
-bool operator!=(ConstructWithSpreadParameters const&,
-                ConstructWithSpreadParameters const&);
+bool operator==(SpreadWithArityParameter const&,
+                SpreadWithArityParameter const&);
+bool operator!=(SpreadWithArityParameter const&,
+                SpreadWithArityParameter const&);
 
-size_t hash_value(ConstructWithSpreadParameters const&);
+size_t hash_value(SpreadWithArityParameter const&);
 
-std::ostream& operator<<(std::ostream&, ConstructWithSpreadParameters const&);
+std::ostream& operator<<(std::ostream&, SpreadWithArityParameter const&);
 
-ConstructWithSpreadParameters const& ConstructWithSpreadParametersOf(
-    Operator const*);
+SpreadWithArityParameter const& SpreadWithArityParameterOf(Operator const*);
 
 // Defines the flags for a JavaScript call forwarding parameters. This
 // is used as parameter by JSCallForwardVarargs operators.
 class CallForwardVarargsParameters final {
  public:
-  CallForwardVarargsParameters(uint32_t start_index,
+  CallForwardVarargsParameters(size_t arity, uint32_t start_index,
                                TailCallMode tail_call_mode)
-      : bit_field_(StartIndexField::encode(start_index) |
+      : bit_field_(ArityField::encode(arity) |
+                   StartIndexField::encode(start_index) |
                    TailCallModeField::encode(tail_call_mode)) {}
 
+  size_t arity() const { return ArityField::decode(bit_field_); }
   uint32_t start_index() const { return StartIndexField::decode(bit_field_); }
   TailCallMode tail_call_mode() const {
     return TailCallModeField::decode(bit_field_);
@@ -138,8 +204,9 @@ class CallForwardVarargsParameters final {
     return p.bit_field_;
   }
 
-  typedef BitField<uint32_t, 0, 30> StartIndexField;
-  typedef BitField<TailCallMode, 31, 1> TailCallModeField;
+  typedef BitField<size_t, 0, 15> ArityField;
+  typedef BitField<uint32_t, 15, 15> StartIndexField;
+  typedef BitField<TailCallMode, 30, 1> TailCallModeField;
 
   uint32_t const bit_field_;
 };
@@ -153,8 +220,9 @@ CallForwardVarargsParameters const& CallForwardVarargsParametersOf(
 // used as a parameter by JSCall operators.
 class CallParameters final {
  public:
-  CallParameters(size_t arity, float frequency, VectorSlotPair const& feedback,
-                 TailCallMode tail_call_mode, ConvertReceiverMode convert_mode)
+  CallParameters(size_t arity, CallFrequency frequency,
+                 VectorSlotPair const& feedback, TailCallMode tail_call_mode,
+                 ConvertReceiverMode convert_mode)
       : bit_field_(ArityField::encode(arity) |
                    ConvertReceiverModeField::encode(convert_mode) |
                    TailCallModeField::encode(tail_call_mode)),
@@ -162,7 +230,7 @@ class CallParameters final {
         feedback_(feedback) {}
 
   size_t arity() const { return ArityField::decode(bit_field_); }
-  float frequency() const { return frequency_; }
+  CallFrequency frequency() const { return frequency_; }
   ConvertReceiverMode convert_mode() const {
     return ConvertReceiverModeField::decode(bit_field_);
   }
@@ -188,7 +256,7 @@ class CallParameters final {
   typedef BitField<TailCallMode, 31, 1> TailCallModeField;
 
   uint32_t const bit_field_;
-  float const frequency_;
+  CallFrequency const frequency_;
   VectorSlotPair const feedback_;
 };
 
@@ -198,29 +266,6 @@ std::ostream& operator<<(std::ostream&, CallParameters const&);
 
 const CallParameters& CallParametersOf(const Operator* op);
 
-// Defines the arity for a JavaScript constructor call with a spread as the last
-// parameters. This is used as a parameter by JSConstructWithSpread
-// operators.
-class CallWithSpreadParameters final {
- public:
-  explicit CallWithSpreadParameters(uint32_t arity) : arity_(arity) {}
-
-  uint32_t arity() const { return arity_; }
-
- private:
-  uint32_t const arity_;
-};
-
-bool operator==(CallWithSpreadParameters const&,
-                CallWithSpreadParameters const&);
-bool operator!=(CallWithSpreadParameters const&,
-                CallWithSpreadParameters const&);
-
-size_t hash_value(CallWithSpreadParameters const&);
-
-std::ostream& operator<<(std::ostream&, CallWithSpreadParameters const&);
-
-CallWithSpreadParameters const& CallWithSpreadParametersOf(Operator const*);
 
 // Defines the arity and the ID for a runtime function call. This is used as a
 // parameter by JSCallRuntime operators.
@@ -356,9 +401,9 @@ const StoreNamedOwnParameters& StoreNamedOwnParametersOf(const Operator* op);
 // Defines the feedback, i.e., vector and index, for storing a data property in
 // an object literal. This is
 // used as a parameter by the JSStoreDataPropertyInLiteral operator.
-class DataPropertyParameters final {
+class FeedbackParameter final {
  public:
-  explicit DataPropertyParameters(VectorSlotPair const& feedback)
+  explicit FeedbackParameter(VectorSlotPair const& feedback)
       : feedback_(feedback) {}
 
   VectorSlotPair const& feedback() const { return feedback_; }
@@ -367,14 +412,14 @@ class DataPropertyParameters final {
   VectorSlotPair const feedback_;
 };
 
-bool operator==(DataPropertyParameters const&, DataPropertyParameters const&);
-bool operator!=(DataPropertyParameters const&, DataPropertyParameters const&);
+bool operator==(FeedbackParameter const&, FeedbackParameter const&);
+bool operator!=(FeedbackParameter const&, FeedbackParameter const&);
 
-size_t hash_value(DataPropertyParameters const&);
+size_t hash_value(FeedbackParameter const&);
 
-std::ostream& operator<<(std::ostream&, DataPropertyParameters const&);
+std::ostream& operator<<(std::ostream&, FeedbackParameter const&);
 
-const DataPropertyParameters& DataPropertyParametersOf(const Operator* op);
+const FeedbackParameter& FeedbackParameterOf(const Operator* op);
 
 // Defines the property of an object for a named access. This is
 // used as a parameter by the JSLoadNamed and JSStoreNamed operators.
@@ -574,6 +619,33 @@ std::ostream& operator<<(std::ostream&, CreateLiteralParameters const&);
 
 const CreateLiteralParameters& CreateLiteralParametersOf(const Operator* op);
 
+class GeneratorStoreParameters final {
+ public:
+  GeneratorStoreParameters(int register_count, SuspendFlags flags)
+      : register_count_(register_count), suspend_flags_(flags) {}
+
+  int register_count() const { return register_count_; }
+  SuspendFlags suspend_flags() const { return suspend_flags_; }
+  SuspendFlags suspend_type() const {
+    return suspend_flags_ & SuspendFlags::kSuspendTypeMask;
+  }
+
+ private:
+  int register_count_;
+  SuspendFlags suspend_flags_;
+};
+
+bool operator==(GeneratorStoreParameters const&,
+                GeneratorStoreParameters const&);
+bool operator!=(GeneratorStoreParameters const&,
+                GeneratorStoreParameters const&);
+
+size_t hash_value(GeneratorStoreParameters const&);
+
+std::ostream& operator<<(std::ostream&, GeneratorStoreParameters const&);
+
+const GeneratorStoreParameters& GeneratorStoreParametersOf(const Operator* op);
+
 BinaryOperationHint BinaryOperationHintOf(const Operator* op);
 
 CompareOperationHint CompareOperationHintOf(const Operator* op);
@@ -587,9 +659,7 @@ class V8_EXPORT_PRIVATE JSOperatorBuilder final
   explicit JSOperatorBuilder(Zone* zone);
 
   const Operator* Equal(CompareOperationHint hint);
-  const Operator* NotEqual(CompareOperationHint hint);
   const Operator* StrictEqual(CompareOperationHint hint);
-  const Operator* StrictNotEqual(CompareOperationHint hint);
   const Operator* LessThan(CompareOperationHint hint);
   const Operator* GreaterThan(CompareOperationHint hint);
   const Operator* LessThanOrEqual(CompareOperationHint hint);
@@ -632,10 +702,10 @@ class V8_EXPORT_PRIVATE JSOperatorBuilder final
   const Operator* CreateLiteralRegExp(Handle<String> constant_pattern,
                                       int literal_flags, int literal_index);
 
-  const Operator* CallForwardVarargs(uint32_t start_index,
+  const Operator* CallForwardVarargs(size_t arity, uint32_t start_index,
                                      TailCallMode tail_call_mode);
   const Operator* Call(
-      size_t arity, float frequency = 0.0f,
+      size_t arity, CallFrequency frequency = CallFrequency(),
       VectorSlotPair const& feedback = VectorSlotPair(),
       ConvertReceiverMode convert_mode = ConvertReceiverMode::kAny,
       TailCallMode tail_call_mode = TailCallMode::kDisallow);
@@ -643,8 +713,11 @@ class V8_EXPORT_PRIVATE JSOperatorBuilder final
   const Operator* CallRuntime(Runtime::FunctionId id);
   const Operator* CallRuntime(Runtime::FunctionId id, size_t arity);
   const Operator* CallRuntime(const Runtime::Function* function, size_t arity);
-  const Operator* Construct(uint32_t arity, float frequency,
-                            VectorSlotPair const& feedback);
+
+  const Operator* ConstructForwardVarargs(size_t arity, uint32_t start_index);
+  const Operator* Construct(uint32_t arity,
+                            CallFrequency frequency = CallFrequency(),
+                            VectorSlotPair const& feedback = VectorSlotPair());
   const Operator* ConstructWithSpread(uint32_t arity);
 
   const Operator* ConvertReceiver(ConvertReceiverMode convert_mode);
@@ -661,12 +734,14 @@ class V8_EXPORT_PRIVATE JSOperatorBuilder final
                                 VectorSlotPair const& feedback);
   const Operator* StoreDataPropertyInLiteral(const VectorSlotPair& feedback);
 
-  const Operator* DeleteProperty(LanguageMode language_mode);
+  const Operator* DeleteProperty();
 
   const Operator* HasProperty();
 
   const Operator* GetSuperConstructor();
 
+  const Operator* CreateGeneratorObject();
+
   const Operator* LoadGlobal(const Handle<Name>& name,
                              const VectorSlotPair& feedback,
                              TypeofMode typeof_mode = NOT_INSIDE_TYPEOF);
@@ -692,7 +767,8 @@ class V8_EXPORT_PRIVATE JSOperatorBuilder final
   const Operator* StoreMessage();
 
   // Used to implement Ignition's SuspendGenerator bytecode.
-  const Operator* GeneratorStore(int register_count);
+  const Operator* GeneratorStore(int register_count,
+                                 SuspendFlags suspend_flags);
 
   // Used to implement Ignition's ResumeGenerator bytecode.
   const Operator* GeneratorRestoreContinuation();
