@@ -22,6 +22,10 @@
 'use strict';
 
 const util = require('util');
+const kCounts = Symbol('counts');
+
+// Track amount of indentation required via `console.group()`.
+const kGroupIndent = Symbol('groupIndent');
 
 function Console(stdout, stderr, ignoreErrors = true) {
   if (!(this instanceof Console)) {
@@ -54,6 +58,11 @@ function Console(stdout, stderr, ignoreErrors = true) {
   prop.value = createWriteErrorHandler(stderr);
   Object.defineProperty(this, '_stderrErrorHandler', prop);
 
+  this[kCounts] = new Map();
+
+  Object.defineProperty(this, kGroupIndent, { writable: true });
+  this[kGroupIndent] = '';
+
   // bind the prototype functions to this Console instance
   var keys = Object.keys(Console.prototype);
   for (var v = 0; v < keys.length; v++) {
@@ -78,7 +87,15 @@ function createWriteErrorHandler(stream) {
   };
 }
 
-function write(ignoreErrors, stream, string, errorhandler) {
+function write(ignoreErrors, stream, string, errorhandler, groupIndent) {
+  if (groupIndent.length !== 0) {
+    if (string.indexOf('\n') !== -1) {
+      string = string.replace(/\n/g, `\n${groupIndent}`);
+    }
+    string = groupIndent + string;
+  }
+  string += '\n';
+
   if (!ignoreErrors) return stream.write(string);
 
   // There may be an error occurring synchronously (e.g. for files or TTYs
@@ -90,6 +107,10 @@ function write(ignoreErrors, stream, string, errorhandler) {
 
     stream.write(string, errorhandler);
   } catch (e) {
+    // console is a debugging utility, so it swallowing errors is not desirable
+    // even in edge cases such as low stack space.
+    if (e.message === 'Maximum call stack size exceeded')
+      throw e;
     // Sorry, there’s no proper way to pass along the error here.
   } finally {
     stream.removeListener('error', noop);
@@ -103,8 +124,9 @@ function write(ignoreErrors, stream, string, errorhandler) {
 Console.prototype.log = function log(...args) {
   write(this._ignoreErrors,
         this._stdout,
-        `${util.format.apply(null, args)}\n`,
-        this._stdoutErrorHandler);
+        util.format.apply(null, args),
+        this._stdoutErrorHandler,
+        this[kGroupIndent]);
 };
 
 
@@ -114,8 +136,9 @@ Console.prototype.info = Console.prototype.log;
 Console.prototype.warn = function warn(...args) {
   write(this._ignoreErrors,
         this._stderr,
-        `${util.format.apply(null, args)}\n`,
-        this._stderrErrorHandler);
+        util.format.apply(null, args),
+        this._stderrErrorHandler,
+        this[kGroupIndent]);
 };
 
 
@@ -126,8 +149,9 @@ Console.prototype.dir = function dir(object, options) {
   options = Object.assign({customInspect: false}, options);
   write(this._ignoreErrors,
         this._stdout,
-        `${util.inspect(object, options)}\n`,
-        this._stdoutErrorHandler);
+        util.inspect(object, options),
+        this._stdoutErrorHandler,
+        this[kGroupIndent]);
 };
 
 
@@ -165,6 +189,56 @@ Console.prototype.assert = function assert(expression, ...args) {
   }
 };
 
+// Defined by: https://console.spec.whatwg.org/#clear
+Console.prototype.clear = function clear() {
+  // It only makes sense to clear if _stdout is a TTY.
+  // Otherwise, do nothing.
+  if (this._stdout.isTTY) {
+    // The require is here intentionally to avoid readline being
+    // required too early when console is first loaded.
+    const { cursorTo, clearScreenDown } = require('readline');
+    cursorTo(this._stdout, 0, 0);
+    clearScreenDown(this._stdout);
+  }
+};
+
+// Defined by: https://console.spec.whatwg.org/#count
+Console.prototype.count = function count(label = 'default') {
+  // Ensures that label is a string, and only things that can be
+  // coerced to strings. e.g. Symbol is not allowed
+  label = `${label}`;
+  const counts = this[kCounts];
+  let count = counts.get(label);
+  if (count === undefined)
+    count = 1;
+  else
+    count++;
+  counts.set(label, count);
+  this.log(`${label}: ${count}`);
+};
+
+// Not yet defined by the https://console.spec.whatwg.org, but
+// proposed to be added and currently implemented by Edge. Having
+// the ability to reset counters is important to help prevent
+// the counter from being a memory leak.
+Console.prototype.countReset = function countReset(label = 'default') {
+  const counts = this[kCounts];
+  counts.delete(`${label}`);
+};
+
+Console.prototype.group = function group(...data) {
+  if (data.length > 0) {
+    this.log(...data);
+  }
+  this[kGroupIndent] += '  ';
+};
+
+Console.prototype.groupCollapsed = Console.prototype.group;
+
+Console.prototype.groupEnd = function groupEnd() {
+  this[kGroupIndent] =
+    this[kGroupIndent].slice(0, this[kGroupIndent].length - 2);
+};
 
 module.exports = new Console(process.stdout, process.stderr);
 module.exports.Console = Console;
