@@ -33,6 +33,7 @@ const { isUint8Array, createPromise, promiseResolve } = process.binding('util');
 const binding = process.binding('fs');
 const fs = exports;
 const Buffer = require('buffer').Buffer;
+const errors = require('internal/errors');
 const Stream = require('stream').Stream;
 const EventEmitter = require('events');
 const FSReqWrap = binding.FSReqWrap;
@@ -172,20 +173,21 @@ function isFd(path) {
 
 // Constructor for file stats.
 function Stats(
-    dev,
-    mode,
-    nlink,
-    uid,
-    gid,
-    rdev,
-    blksize,
-    ino,
-    size,
-    blocks,
-    atim_msec,
-    mtim_msec,
-    ctim_msec,
-    birthtim_msec) {
+  dev,
+  mode,
+  nlink,
+  uid,
+  gid,
+  rdev,
+  blksize,
+  ino,
+  size,
+  blocks,
+  atim_msec,
+  mtim_msec,
+  ctim_msec,
+  birthtim_msec
+) {
   this.dev = dev;
   this.mode = mode;
   this.nlink = nlink;
@@ -520,7 +522,6 @@ function tryStatSync(fd, isUserFd) {
   } finally {
     if (threw && !isUserFd) fs.closeSync(fd);
   }
-  return !threw;
 }
 
 function tryCreateBuffer(size, fd, isUserFd) {
@@ -552,10 +553,11 @@ fs.readFileSync = function(path, options) {
   var isUserFd = isFd(path); // file descriptor ownership
   var fd = isUserFd ? path : fs.openSync(path, options.flag || 'r', 0o666);
 
+  tryStatSync(fd, isUserFd);
   // Use stats array directly to avoid creating an fs.Stats instance just for
   // our internal use.
   var size;
-  if (tryStatSync(fd, isUserFd) && (statValues[1/*mode*/] & S_IFMT) === S_IFREG)
+  if ((statValues[1/*mode*/] & S_IFMT) === S_IFREG)
     size = statValues[8/*size*/];
   else
     size = 0;
@@ -1084,7 +1086,7 @@ if (constants.O_SYMLINK !== undefined) {
         callback(err);
         return;
       }
-      // prefer to return the chmod error, if one occurs,
+      // Prefer to return the chmod error, if one occurs,
       // but still try to close, and report closing errors if they occur.
       fs.fchmod(fd, mode, function(err) {
         fs.close(fd, function(err2) {
@@ -1097,20 +1099,18 @@ if (constants.O_SYMLINK !== undefined) {
   fs.lchmodSync = function(path, mode) {
     var fd = fs.openSync(path, constants.O_WRONLY | constants.O_SYMLINK);
 
-    // prefer to return the chmod error, if one occurs,
+    // Prefer to return the chmod error, if one occurs,
     // but still try to close, and report closing errors if they occur.
-    var err, err2, ret;
+    var ret;
     try {
       ret = fs.fchmodSync(fd, mode);
-    } catch (er) {
-      err = er;
+    } catch (err) {
+      try {
+        fs.closeSync(fd);
+      } catch (ignore) {}
+      throw err;
     }
-    try {
-      fs.closeSync(fd);
-    } catch (er) {
-      err2 = er;
-    }
-    if (err || err2) throw (err || err2);
+    fs.closeSync(fd);
     return ret;
   };
 }
@@ -1285,7 +1285,7 @@ fs.writeFile = function(path, data, options, callback) {
 
   function writeFd(fd, isUserFd) {
     var buffer = isUint8Array(data) ?
-        data : Buffer.from('' + data, options.encoding || 'utf8');
+      data : Buffer.from('' + data, options.encoding || 'utf8');
     var position = /a/.test(flag) ? null : 0;
 
     writeAll(fd, isUserFd, buffer, 0, buffer.length, position, callback);
@@ -1320,7 +1320,7 @@ fs.writeFileSync = function(path, data, options) {
 };
 
 fs.appendFile = function(path, data, options, callback) {
-  callback = maybeCallback(arguments[arguments.length - 1]);
+  callback = maybeCallback(callback || options);
   options = getOptions(options, { encoding: 'utf8', mode: 0o666, flag: 'a' });
 
   // Don't make changes directly on options object
@@ -1357,9 +1357,8 @@ function FSWatcher() {
     if (status < 0) {
       self._handle.close();
       const error = !filename ?
-          errnoException(status, 'Error watching file for changes:') :
-          errnoException(status,
-                         `Error watching file ${filename} for changes:`);
+        errnoException(status, 'Error watching file for changes:') :
+        errnoException(status, `Error watching file ${filename} for changes:`);
       error.filename = filename;
       self.emit('error', error);
     } else {
@@ -1620,7 +1619,7 @@ fs.realpathSync = function realpathSync(p, options) {
     knownHard[base] = true;
   }
 
-  // walk down the path, swapping out linked pathparts for their real
+  // walk down the path, swapping out linked path parts for their real
   // values
   // NB: p.length changes.
   while (pos < p.length) {
@@ -1747,7 +1746,7 @@ fs.realpath = function realpath(p, options, callback) {
     process.nextTick(LOOP);
   }
 
-  // walk down the path, swapping out linked pathparts for their real
+  // walk down the path, swapping out linked path parts for their real
   // values
   function LOOP() {
     // stop if scanned past end of path
@@ -1866,6 +1865,61 @@ fs.mkdtempSync = function(prefix, options) {
 };
 
 
+// Define copyFile() flags.
+Object.defineProperties(fs.constants, {
+  COPYFILE_EXCL: { enumerable: true, value: constants.UV_FS_COPYFILE_EXCL }
+});
+
+
+fs.copyFile = function(src, dest, flags, callback) {
+  if (typeof flags === 'function') {
+    callback = flags;
+    flags = 0;
+  } else if (typeof callback !== 'function') {
+    throw new errors.TypeError('ERR_INVALID_ARG_TYPE', 'callback', 'function');
+  }
+
+  src = getPathFromURL(src);
+
+  if (handleError(src, callback))
+    return;
+
+  if (!nullCheck(src, callback))
+    return;
+
+  dest = getPathFromURL(dest);
+
+  if (handleError(dest, callback))
+    return;
+
+  if (!nullCheck(dest, callback))
+    return;
+
+  src = pathModule._makeLong(src);
+  dest = pathModule._makeLong(dest);
+  flags = flags | 0;
+  const req = new FSReqWrap();
+  req.oncomplete = makeCallback(callback);
+  binding.copyFile(src, dest, flags, req);
+};
+
+
+fs.copyFileSync = function(src, dest, flags) {
+  src = getPathFromURL(src);
+  handleError(src);
+  nullCheck(src);
+
+  dest = getPathFromURL(dest);
+  handleError(dest);
+  nullCheck(dest);
+
+  src = pathModule._makeLong(src);
+  dest = pathModule._makeLong(dest);
+  flags = flags | 0;
+  binding.copyFile(src, dest, flags);
+};
+
+
 var pool;
 
 function allocNewPool(poolSize) {
@@ -1951,10 +2005,11 @@ ReadStream.prototype.open = function() {
 };
 
 ReadStream.prototype._read = function(n) {
-  if (typeof this.fd !== 'number')
+  if (typeof this.fd !== 'number') {
     return this.once('open', function() {
       this._read(n);
     });
+  }
 
   if (this.destroyed)
     return;
@@ -2116,10 +2171,11 @@ WriteStream.prototype._write = function(data, encoding, cb) {
   if (!(data instanceof Buffer))
     return this.emit('error', new Error('Invalid data'));
 
-  if (typeof this.fd !== 'number')
+  if (typeof this.fd !== 'number') {
     return this.once('open', function() {
       this._write(data, encoding, cb);
     });
+  }
 
   var self = this;
   fs.write(this.fd, data, 0, data.length, this.pos, function(er, bytes) {
@@ -2151,10 +2207,11 @@ function writev(fd, chunks, position, callback) {
 
 
 WriteStream.prototype._writev = function(data, cb) {
-  if (typeof this.fd !== 'number')
+  if (typeof this.fd !== 'number') {
     return this.once('open', function() {
       this._writev(data, cb);
     });
+  }
 
   const self = this;
   const len = data.length;
