@@ -14,9 +14,7 @@
     process._eventsCount = 0;
 
     const origProcProto = Object.getPrototypeOf(process);
-    Object.setPrototypeOf(process, Object.create(EventEmitter.prototype, {
-      constructor: Object.getOwnPropertyDescriptor(origProcProto, 'constructor')
-    }));
+    Object.setPrototypeOf(origProcProto, EventEmitter.prototype);
 
     EventEmitter.call(process);
 
@@ -28,25 +26,57 @@
     setupProcessICUVersions();
 
     setupGlobalVariables();
-    if (!process._noBrowserGlobals) {
+    const browserGlobals = !process._noBrowserGlobals;
+    if (browserGlobals) {
       setupGlobalTimeouts();
       setupGlobalConsole();
     }
 
     const _process = NativeModule.require('internal/process');
+    const perf = process.binding('performance');
+    const {
+      NODE_PERFORMANCE_MILESTONE_BOOTSTRAP_COMPLETE,
+      NODE_PERFORMANCE_MILESTONE_THIRD_PARTY_MAIN_START,
+      NODE_PERFORMANCE_MILESTONE_THIRD_PARTY_MAIN_END,
+      NODE_PERFORMANCE_MILESTONE_CLUSTER_SETUP_START,
+      NODE_PERFORMANCE_MILESTONE_CLUSTER_SETUP_END,
+      NODE_PERFORMANCE_MILESTONE_MODULE_LOAD_START,
+      NODE_PERFORMANCE_MILESTONE_MODULE_LOAD_END,
+      NODE_PERFORMANCE_MILESTONE_PRELOAD_MODULE_LOAD_START,
+      NODE_PERFORMANCE_MILESTONE_PRELOAD_MODULE_LOAD_END
+    } = perf.constants;
 
     _process.setup_hrtime();
+    _process.setup_performance();
     _process.setup_cpuUsage();
     _process.setupMemoryUsage();
     _process.setupConfig(NativeModule._source);
     NativeModule.require('internal/process/warning').setup();
     NativeModule.require('internal/process/next_tick').setup();
     NativeModule.require('internal/process/stdio').setup();
+    if (browserGlobals) {
+      // Instantiate eagerly in case the first call is under stack overflow
+      // conditions where instantiation doesn't work.
+      const console = global.console;
+      console.assert;
+      console.clear;
+      console.count;
+      console.countReset;
+      console.dir;
+      console.error;
+      console.log;
+      console.time;
+      console.timeEnd;
+      console.trace;
+      console.warn;
+    }
     _process.setupKillAndExit();
     _process.setupSignalHandlers();
     if (global.__coverage__)
       NativeModule.require('internal/process/write-coverage').setup();
 
+    NativeModule.require('internal/inspector_async_hook').setup();
+
     // Do not initialize channel in debugger agent, it deletes env variable
     // and the main thread won't see it.
     if (process.argv[1] !== '--debug-agent')
@@ -79,6 +109,13 @@
         'DeprecationWarning', 'DEP0062', startup, true);
     }
 
+    if (process.binding('config').experimentalModules) {
+      process.emitWarning(
+        'The ESM module loader is experimental.',
+        'ExperimentalWarning', undefined);
+    }
+
+
     // There are various modes that Node can run in. The most common two
     // are running from a script and running the REPL - but there are a few
     // others like the debugger or running --eval arguments. Here we decide
@@ -89,7 +126,9 @@
       // one to drop a file lib/_third_party_main.js into the build
       // directory which will be executed instead of Node's normal loading.
       process.nextTick(function() {
+        perf.markMilestone(NODE_PERFORMANCE_MILESTONE_THIRD_PARTY_MAIN_START);
         NativeModule.require('_third_party_main');
+        perf.markMilestone(NODE_PERFORMANCE_MILESTONE_THIRD_PARTY_MAIN_END);
       });
 
     } else if (process.argv[1] === 'inspect' || process.argv[1] === 'debug') {
@@ -104,14 +143,6 @@
         NativeModule.require('node-inspect/lib/_inspect').start();
       });
 
-    } else if (process.argv[1] === '--remote_debugging_server') {
-      // Start the debugging server
-      NativeModule.require('internal/inspector/remote_debugging_server');
-
-    } else if (process.argv[1] === '--debug-agent') {
-      // Start the debugger agent
-      NativeModule.require('_debug_agent').start();
-
     } else if (process.profProcess) {
       NativeModule.require('internal/v8_prof_processor');
 
@@ -122,22 +153,30 @@
       // channel. This needs to be done before any user code gets executed
       // (including preload modules).
       if (process.argv[1] && process.env.NODE_UNIQUE_ID) {
+        perf.markMilestone(NODE_PERFORMANCE_MILESTONE_CLUSTER_SETUP_START);
         const cluster = NativeModule.require('cluster');
         cluster._setupWorker();
-
+        perf.markMilestone(NODE_PERFORMANCE_MILESTONE_CLUSTER_SETUP_END);
         // Make sure it's not accidentally inherited by child processes.
         delete process.env.NODE_UNIQUE_ID;
       }
 
       if (process._eval != null && !process._forceRepl) {
+        perf.markMilestone(NODE_PERFORMANCE_MILESTONE_MODULE_LOAD_START);
+        perf.markMilestone(NODE_PERFORMANCE_MILESTONE_MODULE_LOAD_END);
         // User passed '-e' or '--eval' arguments to Node without '-i' or
         // '--interactive'
+
+        perf.markMilestone(
+          NODE_PERFORMANCE_MILESTONE_PRELOAD_MODULE_LOAD_START);
         preloadModules();
+        perf.markMilestone(NODE_PERFORMANCE_MILESTONE_PRELOAD_MODULE_LOAD_END);
 
         const internalModule = NativeModule.require('internal/module');
         internalModule.addBuiltinLibsToObject(global);
         evalScript('[eval]');
       } else if (process.argv[1] && process.argv[1] !== '-') {
+        perf.markMilestone(NODE_PERFORMANCE_MILESTONE_MODULE_LOAD_START);
         // make process.argv[1] into a full path
         const path = NativeModule.require('path');
         process.argv[1] = path.resolve(process.argv[1]);
@@ -153,11 +192,21 @@
           checkScriptSyntax(source, filename);
           process.exit(0);
         }
-
+        perf.markMilestone(NODE_PERFORMANCE_MILESTONE_MODULE_LOAD_END);
+        perf.markMilestone(
+          NODE_PERFORMANCE_MILESTONE_PRELOAD_MODULE_LOAD_START);
         preloadModules();
+        perf.markMilestone(
+          NODE_PERFORMANCE_MILESTONE_PRELOAD_MODULE_LOAD_END);
         Module.runMain();
       } else {
+        perf.markMilestone(NODE_PERFORMANCE_MILESTONE_MODULE_LOAD_START);
+        perf.markMilestone(NODE_PERFORMANCE_MILESTONE_MODULE_LOAD_END);
+        perf.markMilestone(
+          NODE_PERFORMANCE_MILESTONE_PRELOAD_MODULE_LOAD_START);
         preloadModules();
+        perf.markMilestone(
+          NODE_PERFORMANCE_MILESTONE_PRELOAD_MODULE_LOAD_END);
         // If -i or --interactive were passed, or stdin is a TTY.
         if (process._forceRepl || NativeModule.require('tty').isatty(0)) {
           // REPL
@@ -201,6 +250,7 @@
         }
       }
     }
+    perf.markMilestone(NODE_PERFORMANCE_MILESTONE_BOOTSTRAP_COMPLETE);
   }
 
   function setupProcessObject() {
@@ -282,11 +332,15 @@
         return console;
       }
     });
+    setupInspectorCommandLineAPI();
   }
 
   function installInspectorConsole(globalConsole) {
     const wrappedConsole = NativeModule.require('console');
     const inspector = process.binding('inspector');
+    if (!inspector.consoleCall) {
+      return wrappedConsole;
+    }
     const config = {};
     for (const key of Object.keys(wrappedConsole)) {
       if (!globalConsole.hasOwnProperty(key))
@@ -307,12 +361,28 @@
     return wrappedConsole;
   }
 
+  function setupInspectorCommandLineAPI() {
+    const { addCommandLineAPI } = process.binding('inspector');
+    if (!addCommandLineAPI) return;
+
+    const Module = NativeModule.require('module');
+    const { makeRequireFunction } = NativeModule.require('internal/module');
+    const path = NativeModule.require('path');
+    const cwd = tryGetCwd(path);
+
+    const consoleAPIModule = new Module('<inspector console>');
+    consoleAPIModule.paths =
+        Module._nodeModulePaths(cwd).concat(Module.globalPaths);
+
+    addCommandLineAPI('require', makeRequireFunction(consoleAPIModule));
+  }
+
   function setupProcessFatal() {
     const async_wrap = process.binding('async_wrap');
     // Arrays containing hook flags and ids for async_hook calls.
     const { async_hook_fields, async_uid_fields } = async_wrap;
     // Internal functions needed to manipulate the stack.
-    const { clearIdStack, popAsyncIds } = async_wrap;
+    const { clearIdStack, asyncIdStackSize } = async_wrap;
     const { kAfter, kCurrentAsyncId, kInitTriggerId } = async_wrap.constants;
 
     process._fatalException = function(er) {
@@ -349,8 +419,7 @@
           do {
             NativeModule.require('async_hooks').emitAfter(
               async_uid_fields[kCurrentAsyncId]);
-          // popAsyncIds() returns true if there are more ids on the stack.
-          } while (popAsyncIds(async_uid_fields[kCurrentAsyncId]));
+          } while (asyncIdStackSize() > 0);
         // Or completely empty the id stack.
         } else {
           clearIdStack();
@@ -369,47 +438,26 @@
     // of possible types.
     const versionTypes = icu.getVersion().split(',');
 
-    function makeGetter(name) {
-      return () => {
-        // With an argument, getVersion(type) returns
-        // the actual version string.
-        const version = icu.getVersion(name);
-        // Replace the current getter with a new property.
-        delete process.versions[name];
-        Object.defineProperty(process.versions, name, {
-          value: version,
-          writable: false,
-          enumerable: true
-        });
-        return version;
-      };
-    }
-
     for (var n = 0; n < versionTypes.length; n++) {
       var name = versionTypes[n];
+      const version = icu.getVersion(name);
       Object.defineProperty(process.versions, name, {
-        configurable: true,
+        writable: false,
         enumerable: true,
-        get: makeGetter(name)
+        value: version
       });
     }
   }
 
   function tryGetCwd(path) {
-    var threw = true;
-    var cwd;
     try {
-      cwd = process.cwd();
-      threw = false;
-    } finally {
-      if (threw) {
-        // getcwd(3) can fail if the current working directory has been deleted.
-        // Fall back to the directory name of the (absolute) executable path.
-        // It's not really correct but what are the alternatives?
-        return path.dirname(process.execPath);
-      }
+      return process.cwd();
+    } catch (ex) {
+      // getcwd(3) can fail if the current working directory has been deleted.
+      // Fall back to the directory name of the (absolute) executable path.
+      // It's not really correct but what are the alternatives?
+      return path.dirname(process.execPath);
     }
-    return cwd;
   }
 
   function evalScript(name) {
@@ -478,6 +526,11 @@
   NativeModule._source = process.binding('natives');
   NativeModule._cache = {};
 
+  const config = process.binding('config');
+
+  if (!config.exposeHTTP2)
+    delete NativeModule._source.http2;
+
   NativeModule.require = function(id) {
     if (id === 'native_module') {
       return NativeModule;
@@ -516,8 +569,6 @@
     return NativeModule._source.hasOwnProperty(id);
   };
 
-  const config = process.binding('config');
-
   if (config.exposeInternals) {
     NativeModule.nonInternalExists = NativeModule.exists;
 
