@@ -448,10 +448,14 @@ function setupChannel(target, channel) {
 
   var decoder = new StringDecoder('utf8');
   var jsonBuffer = '';
+  var pendingHandle = null;
   channel.buffering = false;
   channel.onread = function(nread, pool, recvHandle) {
     // TODO(bnoordhuis) Check that nread > 0.
     if (pool) {
+      if (recvHandle)
+        pendingHandle = recvHandle;
+
       // Linebreak is used as a message end sign
       var chunks = decoder.write(pool).split('\n');
       var numCompleteChunks = chunks.length - 1;
@@ -471,10 +475,12 @@ function setupChannel(target, channel) {
         // read because SCM_RIGHTS messages don't get coalesced. Make sure
         // that we deliver the handle with the right message however.
         if (isInternal(message)) {
-          if (message.cmd === 'NODE_HANDLE')
-            handleMessage(message, recvHandle, true);
-          else
+          if (message.cmd === 'NODE_HANDLE') {
+            handleMessage(message, pendingHandle, true);
+            pendingHandle = null;
+          } else {
             handleMessage(message, undefined, true);
+          }
         } else {
           handleMessage(message, undefined, false);
         }
