@@ -9,8 +9,9 @@ class FreeList {
   }
 
   alloc() {
-    return this.list.length ? this.list.pop() :
-                              this.ctor.apply(this, arguments);
+    return this.list.length ?
+      this.list.pop() :
+      this.ctor.apply(this, arguments);
   }
 
   free(obj) {
