@@ -90,7 +90,7 @@ function setupProcessWarnings() {
       if (isDeprecation && process.noDeprecation) return;
       const trace = process.traceProcessWarnings ||
                     (isDeprecation && process.traceDeprecation);
-      let msg = `${prefix}`;
+      var msg = `${prefix}`;
       if (warning.code)
         msg += `[${warning.code}] `;
       if (trace && warning.stack) {
