@@ -15,6 +15,11 @@ module.exports.createInternalRepl = createRepl;
 // The debounce is to guard against code pasted into the REPL.
 const kDebounceHistoryMS = 15;
 
+function _writeToOutput(repl, message) {
+  repl._writeToOutput(message);
+  repl._refreshLine();
+}
+
 function createRepl(env, opts, cb) {
   if (typeof opts === 'function') {
     cb = opts;
@@ -80,9 +85,8 @@ function setupHistory(repl, historyPath, oldHistoryPath, ready) {
     try {
       historyPath = path.join(os.homedir(), '.node_repl_history');
     } catch (err) {
-      repl._writeToOutput('\nError: Could not get the home directory.\n' +
-                          'REPL session history will not be persisted.\n');
-      repl._refreshLine();
+      _writeToOutput(repl, '\nError: Could not get the home directory.\n' +
+                     'REPL session history will not be persisted.\n');
 
       debug(err.stack);
       repl._historyPrev = _replHistoryMessage;
@@ -103,9 +107,8 @@ function setupHistory(repl, historyPath, oldHistoryPath, ready) {
     if (err) {
       // Cannot open history file.
       // Don't crash, just don't persist history.
-      repl._writeToOutput('\nError: Could not open history file.\n' +
-                          'REPL session history will not be persisted.\n');
-      repl._refreshLine();
+      _writeToOutput(repl, '\nError: Could not open history file.\n' +
+                           'REPL session history will not be persisted.\n');
       debug(err.stack);
 
       repl._historyPrev = _replHistoryMessage;
@@ -132,18 +135,13 @@ function setupHistory(repl, historyPath, oldHistoryPath, ready) {
     } else if (oldHistoryPath === historyPath) {
       // If pre-v3.0, the user had set NODE_REPL_HISTORY_FILE to
       // ~/.node_repl_history, warn the user about it and proceed.
-      repl._writeToOutput(
-          '\nThe old repl history file has the same name and location as ' +
+      _writeToOutput(
+        repl,
+        '\nThe old repl history file has the same name and location as ' +
           `the new one i.e., ${historyPath} and is empty.\nUsing it as is.\n`);
-      repl._refreshLine();
 
     } else if (oldHistoryPath) {
-      // Grab data from the older pre-v3.0 JSON NODE_REPL_HISTORY_FILE format.
-      repl._writeToOutput(
-          '\nConverting old JSON repl history to line-separated history.\n' +
-          `The new repl history file can be found at ${historyPath}.\n`);
-      repl._refreshLine();
-
+      let threw = false;
       try {
         // Pre-v3.0, repl history was stored as JSON.
         // Try and convert it to line separated history.
@@ -152,15 +150,31 @@ function setupHistory(repl, historyPath, oldHistoryPath, ready) {
         // Only attempt to use the history if there was any.
         if (oldReplJSONHistory) repl.history = JSON.parse(oldReplJSONHistory);
 
-        if (!Array.isArray(repl.history)) {
-          throw new Error('Expected array, got ' + typeof repl.history);
+        if (Array.isArray(repl.history)) {
+          repl.history = repl.history.slice(0, repl.historySize);
+        } else {
+          threw = true;
+          _writeToOutput(
+            repl,
+            '\nError: The old history file data has to be an Array.\n' +
+              'REPL session history will not be persisted.\n');
         }
-        repl.history = repl.history.slice(0, repl.historySize);
       } catch (err) {
-        if (err.code !== 'ENOENT') {
-          return ready(
-            new Error(`Could not parse history data in ${oldHistoryPath}.`));
-        }
+        // Cannot open or parse history file.
+        // Don't crash, just don't persist history.
+        threw = true;
+        const type = err instanceof SyntaxError ? 'parse' : 'open';
+        _writeToOutput(repl, `\nError: Could not ${type} old history file.\n` +
+                            'REPL session history will not be persisted.\n');
+      }
+      if (!threw) {
+        // Grab data from the older pre-v3.0 JSON NODE_REPL_HISTORY_FILE format.
+        _writeToOutput(
+          repl,
+          '\nConverted old JSON repl history to line-separated history.\n' +
+            `The new repl history file can be found at ${historyPath}.\n`);
+      } else {
+        repl.history = [];
       }
     }
 
@@ -223,12 +237,12 @@ function setupHistory(repl, historyPath, oldHistoryPath, ready) {
 
 function _replHistoryMessage() {
   if (this.history.length === 0) {
-    this._writeToOutput(
-        '\nPersistent history support disabled. ' +
+    _writeToOutput(
+      this,
+      '\nPersistent history support disabled. ' +
         'Set the NODE_REPL_HISTORY environment\nvariable to ' +
         'a valid, user-writable path to enable.\n'
     );
-    this._refreshLine();
   }
   this._historyPrev = Interface.prototype._historyPrev;
   return this._historyPrev();
