@@ -24,6 +24,7 @@
 const NativeModule = require('native_module');
 const util = require('util');
 const internalModule = require('internal/module');
+const { getURLFromFilePath } = require('internal/url');
 const vm = require('vm');
 const assert = require('assert').ok;
 const fs = require('fs');
@@ -32,6 +33,14 @@ const path = require('path');
 const internalModuleReadFile = process.binding('fs').internalModuleReadFile;
 const internalModuleStat = process.binding('fs').internalModuleStat;
 const preserveSymlinks = !!process.binding('config').preserveSymlinks;
+const experimentalModules = !!process.binding('config').experimentalModules;
+
+const errors = require('internal/errors');
+
+const Loader = require('internal/loader/Loader');
+const ModuleJob = require('internal/loader/ModuleJob');
+const { createDynamicModule } = require('internal/loader/ModuleWrap');
+const ESMLoader = new Loader();
 
 function stat(filename) {
   filename = path._makeLong(filename);
@@ -46,15 +55,17 @@ function stat(filename) {
 }
 stat.cache = null;
 
+function updateChildren(parent, child, scan) {
+  var children = parent && parent.children;
+  if (children && !(scan && children.includes(child)))
+    children.push(child);
+}
 
 function Module(id, parent) {
   this.id = id;
   this.exports = {};
   this.parent = parent;
-  if (parent && parent.children) {
-    parent.children.push(this);
-  }
-
+  updateChildren(parent, this, false);
   this.filename = null;
   this.loaded = false;
   this.children = [];
@@ -434,10 +445,40 @@ Module._load = function(request, parent, isMain) {
     debug('Module._load REQUEST %s parent: %s', request, parent.id);
   }
 
-  var filename = Module._resolveFilename(request, parent, isMain);
+  var filename = null;
+
+  if (isMain) {
+    let err;
+    try {
+      filename = Module._resolveFilename(request, parent, isMain);
+    } catch (e) {
+      // try to keep stack
+      e.stack;
+      err = e;
+    }
+    if (experimentalModules) {
+      if (filename === null || /\.mjs$/.test(filename)) {
+        try {
+          ESMLoader.import(request).catch((e) => {
+            console.error(e);
+            process.exit(1);
+          });
+          return;
+        } catch (e) {
+          // well, it isn't ESM
+        }
+      }
+    }
+    if (err) {
+      throw err;
+    }
+  } else {
+    filename = Module._resolveFilename(request, parent, isMain);
+  }
 
   var cachedModule = Module._cache[filename];
   if (cachedModule) {
+    updateChildren(parent, cachedModule, true);
     return cachedModule.exports;
   }
 
@@ -446,6 +487,7 @@ Module._load = function(request, parent, isMain) {
     return NativeModule.require(filename);
   }
 
+  // Don't call updateChildren(), Module constructor already does.
   var module = new Module(filename, parent);
 
   if (isMain) {
@@ -502,6 +544,19 @@ Module.prototype.load = function(filename) {
   if (!Module._extensions[extension]) extension = '.js';
   Module._extensions[extension](this, filename);
   this.loaded = true;
+
+  if (experimentalModules) {
+    const url = getURLFromFilePath(filename);
+    if (ESMLoader.moduleMap.has(`${url}`) !== true) {
+      const ctx = createDynamicModule(['default'], url);
+      ctx.reflect.exports.default.set(this.exports);
+      ESMLoader.moduleMap.set(`${url}`,
+                              new ModuleJob(ESMLoader, ctx.module));
+    } else {
+      ESMLoader.moduleMap.get(`${url}`).moduleProvider.finish(
+        Module._cache[filename]);
+    }
+  }
 };
 
 
@@ -598,6 +653,11 @@ Module._extensions['.node'] = function(module, filename) {
   return process.dlopen(module, path._makeLong(filename));
 };
 
+if (experimentalModules) {
+  Module._extensions['.mjs'] = function(module, filename) {
+    throw new errors.Error('ERR_REQUIRE_ESM', filename);
+  };
+}
 
 // bootstrap main module.
 Module.runMain = function() {
