@@ -35,6 +35,7 @@ function normalizeStringWin32(path, allowAboveRoot) {
   var lastSlash = -1;
   var dots = 0;
   var code;
+  var isAboveRoot = false;
   for (var i = 0; i <= path.length; ++i) {
     if (i < path.length)
       code = path.charCodeAt(i);
@@ -46,7 +47,7 @@ function normalizeStringWin32(path, allowAboveRoot) {
       if (lastSlash === i - 1 || dots === 1) {
         // NOOP
       } else if (lastSlash !== i - 1 && dots === 2) {
-        if (res.length < 2 ||
+        if (res.length < 2 || !isAboveRoot ||
             res.charCodeAt(res.length - 1) !== 46/*.*/ ||
             res.charCodeAt(res.length - 2) !== 46/*.*/) {
           if (res.length > 2) {
@@ -63,12 +64,14 @@ function normalizeStringWin32(path, allowAboveRoot) {
                 res = res.slice(0, j);
               lastSlash = i;
               dots = 0;
+              isAboveRoot = false;
               continue;
             }
           } else if (res.length === 2 || res.length === 1) {
             res = '';
             lastSlash = i;
             dots = 0;
+            isAboveRoot = false;
             continue;
           }
         }
@@ -77,12 +80,14 @@ function normalizeStringWin32(path, allowAboveRoot) {
             res += '\\..';
           else
             res = '..';
+          isAboveRoot = true;
         }
       } else {
         if (res.length > 0)
           res += '\\' + path.slice(lastSlash + 1, i);
         else
           res = path.slice(lastSlash + 1, i);
+        isAboveRoot = false;
       }
       lastSlash = i;
       dots = 0;
@@ -101,6 +106,7 @@ function normalizeStringPosix(path, allowAboveRoot) {
   var lastSlash = -1;
   var dots = 0;
   var code;
+  var isAboveRoot = false;
   for (var i = 0; i <= path.length; ++i) {
     if (i < path.length)
       code = path.charCodeAt(i);
@@ -112,7 +118,7 @@ function normalizeStringPosix(path, allowAboveRoot) {
       if (lastSlash === i - 1 || dots === 1) {
         // NOOP
       } else if (lastSlash !== i - 1 && dots === 2) {
-        if (res.length < 2 ||
+        if (res.length < 2 || !isAboveRoot ||
             res.charCodeAt(res.length - 1) !== 46/*.*/ ||
             res.charCodeAt(res.length - 2) !== 46/*.*/) {
           if (res.length > 2) {
@@ -129,12 +135,14 @@ function normalizeStringPosix(path, allowAboveRoot) {
                 res = res.slice(0, j);
               lastSlash = i;
               dots = 0;
+              isAboveRoot = false;
               continue;
             }
           } else if (res.length === 2 || res.length === 1) {
             res = '';
             lastSlash = i;
             dots = 0;
+            isAboveRoot = false;
             continue;
           }
         }
@@ -143,12 +151,14 @@ function normalizeStringPosix(path, allowAboveRoot) {
             res += '/..';
           else
             res = '..';
+          isAboveRoot = true;
         }
       } else {
         if (res.length > 0)
           res += '/' + path.slice(lastSlash + 1, i);
         else
           res = path.slice(lastSlash + 1, i);
+        isAboveRoot = false;
       }
       lastSlash = i;
       dots = 0;
@@ -788,7 +798,9 @@ const win32 = {
         }
       }
     } else if (code === 47/*/*/ || code === 92/*\*/) {
-      return path[0];
+      // `path` contains just a path separator, exit early to avoid
+      // unnecessary work
+      return path;
     }
 
     for (var i = len - 1; i >= offset; --i) {
@@ -906,6 +918,7 @@ const win32 = {
 
   extname: function extname(path) {
     assertPath(path);
+    var start = 0;
     var startDot = -1;
     var startPart = 0;
     var end = -1;
@@ -913,7 +926,20 @@ const win32 = {
     // Track the state of characters (if any) we see before our first dot and
     // after any path separator we find
     var preDotState = 0;
-    for (var i = path.length - 1; i >= 0; --i) {
+
+    // Check for a drive letter prefix so as not to mistake the following
+    // path separator as an extra separator at the end of the path that can be
+    // disregarded
+    if (path.length >= 2) {
+      const code = path.charCodeAt(0);
+      if (path.charCodeAt(1) === 58/*:*/ &&
+          ((code >= 65/*A*/ && code <= 90/*Z*/) ||
+           (code >= 97/*a*/ && code <= 122/*z*/))) {
+        start = startPart = 2;
+      }
+    }
+
+    for (var i = path.length - 1; i >= start; --i) {
       const code = path.charCodeAt(i);
       if (code === 47/*/*/ || code === 92/*\*/) {
         // If we reached a path separator that was not part of a set of path
@@ -977,15 +1003,12 @@ const win32 = {
     var len = path.length;
     var rootEnd = 0;
     var code = path.charCodeAt(0);
-    var isAbsolute = false;
 
     // Try to match a root
     if (len > 1) {
       if (code === 47/*/*/ || code === 92/*\*/) {
         // Possible UNC root
 
-        isAbsolute = true;
-
         code = path.charCodeAt(1);
         rootEnd = 1;
         if (code === 47/*/*/ || code === 92/*\*/) {
@@ -1041,16 +1064,15 @@ const win32 = {
               if (len === 3) {
                 // `path` contains just a drive root, exit early to avoid
                 // unnecessary work
-                ret.root = ret.dir = path.slice(0, 3);
+                ret.root = ret.dir = path;
                 return ret;
               }
-              isAbsolute = true;
               rootEnd = 3;
             }
           } else {
             // `path` contains just a drive root, exit early to avoid
             // unnecessary work
-            ret.root = ret.dir = path.slice(0, 2);
+            ret.root = ret.dir = path;
             return ret;
           }
         }
@@ -1058,7 +1080,7 @@ const win32 = {
     } else if (code === 47/*/*/ || code === 92/*\*/) {
       // `path` contains just a path separator, exit early to avoid
       // unnecessary work
-      ret.root = ret.dir = path[0];
+      ret.root = ret.dir = path;
       return ret;
     }
 
@@ -1066,7 +1088,7 @@ const win32 = {
       ret.root = path.slice(0, rootEnd);
 
     var startDot = -1;
-    var startPart = 0;
+    var startPart = rootEnd;
     var end = -1;
     var matchedSlash = true;
     var i = path.length - 1;
@@ -1115,26 +1137,21 @@ const win32 = {
          startDot === end - 1 &&
          startDot === startPart + 1)) {
       if (end !== -1) {
-        if (startPart === 0 && isAbsolute)
-          ret.base = ret.name = path.slice(rootEnd, end);
-        else
-          ret.base = ret.name = path.slice(startPart, end);
+        ret.base = ret.name = path.slice(startPart, end);
       }
     } else {
-      if (startPart === 0 && isAbsolute) {
-        ret.name = path.slice(rootEnd, startDot);
-        ret.base = path.slice(rootEnd, end);
-      } else {
-        ret.name = path.slice(startPart, startDot);
-        ret.base = path.slice(startPart, end);
-      }
+      ret.name = path.slice(startPart, startDot);
+      ret.base = path.slice(startPart, end);
       ret.ext = path.slice(startDot, end);
     }
 
-    if (startPart > 0)
+    // If the directory is the root, use the entire root as the `dir` including
+    // the trailing slash if any (`C:\abc` -> `C:\`). Otherwise, strip out the
+    // trailing slash (`C:\abc\def` -> `C:\abc`).
+    if (startPart > 0 && startPart !== rootEnd)
       ret.dir = path.slice(0, startPart - 1);
-    else if (isAbsolute)
-      ret.dir = path.slice(0, rootEnd);
+    else
+      ret.dir = ret.root;
 
     return ret;
   },
