@@ -46,9 +46,10 @@ const {
   kClearScreenDown
 } = CSI;
 
+const now = process.binding('timer_wrap').Timer.now;
+
 const kHistorySize = 30;
 const kMincrlfDelay = 100;
-const kMaxcrlfDelay = 2000;
 // \r\n, \n, or \r followed by something other than \n
 const lineEnding = /\r?\n|\r(?!\n)/;
 
@@ -120,8 +121,8 @@ function Interface(input, output, completer, terminal) {
   this.input = input;
   this.historySize = historySize;
   this.removeHistoryDuplicates = !!removeHistoryDuplicates;
-  this.crlfDelay = Math.max(kMincrlfDelay,
-                            Math.min(kMaxcrlfDelay, crlfDelay >>> 0));
+  this.crlfDelay = crlfDelay ?
+    Math.max(kMincrlfDelay, crlfDelay) : kMincrlfDelay;
 
   // Check arity, 2 - for async, 1 for sync
   if (typeof completer === 'function') {
@@ -396,7 +397,7 @@ Interface.prototype._normalWrite = function(b) {
   }
   var string = this._decoder.write(b);
   if (this._sawReturnAt &&
-      Date.now() - this._sawReturnAt <= this.crlfDelay) {
+      now() - this._sawReturnAt <= this.crlfDelay) {
     string = string.replace(/^\n/, '');
     this._sawReturnAt = 0;
   }
@@ -409,7 +410,7 @@ Interface.prototype._normalWrite = function(b) {
     this._line_buffer = null;
   }
   if (newPartContainsEnding) {
-    this._sawReturnAt = string.endsWith('\r') ? Date.now() : 0;
+    this._sawReturnAt = string.endsWith('\r') ? now() : 0;
 
     // got one or more newlines; process into "line" events
     var lines = string.split(lineEnding);
@@ -472,7 +473,7 @@ Interface.prototype._tabComplete = function(lastKeypressWasTab) {
           maxColumns = 1;
         }
         var group = [];
-        for (var i = 0, compLen = completions.length; i < compLen; i++) {
+        for (var i = 0; i < completions.length; i++) {
           var c = completions[i];
           if (c === '') {
             handleGroup(self, group, width, maxColumns);
@@ -511,7 +512,7 @@ function handleGroup(self, group, width, maxColumns) {
       var item = group[idx];
       self._writeToOutput(item);
       if (col < maxColumns - 1) {
-        for (var s = 0, itemLen = item.length; s < width - itemLen; s++) {
+        for (var s = 0; s < width - item.length; s++) {
           self._writeToOutput(' ');
         }
       }
@@ -901,14 +902,14 @@ Interface.prototype._ttyWrite = function(s, key) {
 
     switch (key.name) {
       case 'return':  // carriage return, i.e. \r
-        this._sawReturnAt = Date.now();
+        this._sawReturnAt = now();
         this._line();
         break;
 
       case 'enter':
         // When key interval > crlfDelay
         if (this._sawReturnAt === 0 ||
-            Date.now() - this._sawReturnAt > this.crlfDelay) {
+            now() - this._sawReturnAt > this.crlfDelay) {
           this._line();
         }
         this._sawReturnAt = 0;
