@@ -34,16 +34,13 @@ const kOnTimeout = TimerWrap.kOnTimeout | 0;
 const initTriggerId = async_hooks.initTriggerId;
 // Two arrays that share state between C++ and JS.
 const { async_hook_fields, async_uid_fields } = async_wrap;
-// Used to change the state of the async id stack.
-const { pushAsyncIds, popAsyncIds } = async_wrap;
 // The needed emit*() functions.
 const { emitInit, emitBefore, emitAfter, emitDestroy } = async_hooks;
 // Grab the constants necessary for working with internal arrays.
-const { kInit, kBefore, kAfter, kDestroy, kAsyncUidCntr } =
-    async_wrap.constants;
+const { kInit, kDestroy, kAsyncUidCntr } = async_wrap.constants;
 // Symbols for storing async id state.
 const async_id_symbol = Symbol('asyncId');
-const trigger_id_symbol = Symbol('triggerId');
+const trigger_id_symbol = Symbol('triggerAsyncId');
 
 // Timeout values > TIMEOUT_MAX are set to 1.
 const TIMEOUT_MAX = 2147483647; // 2^31-1
@@ -149,22 +146,6 @@ exports._unrefActive = function(item) {
 };
 
 
-function timerEmitBefore(asyncId, triggerId) {
-  if (async_hook_fields[kBefore] > 0)
-    emitBefore(asyncId, triggerId);
-  else
-    pushAsyncIds(asyncId, triggerId);
-}
-
-
-function timerEmitAfter(asyncId) {
-  if (async_hook_fields[kAfter] > 0)
-    emitAfter(asyncId);
-  else
-    popAsyncIds(asyncId);
-}
-
-
 // The underlying logic for scheduling or re-scheduling a timer.
 //
 // Appends a timer onto the end of an existing timers list, or creates a new
@@ -315,17 +296,17 @@ function listOnTimeout() {
 function tryOnTimeout(timer, list) {
   timer._called = true;
   const timerAsyncId = (typeof timer[async_id_symbol] === 'number') ?
-      timer[async_id_symbol] : null;
+    timer[async_id_symbol] : null;
   var threw = true;
   if (timerAsyncId !== null)
-    timerEmitBefore(timerAsyncId, timer[trigger_id_symbol]);
+    emitBefore(timerAsyncId, timer[trigger_id_symbol]);
   try {
     ontimeout(timer);
     threw = false;
   } finally {
     if (timerAsyncId !== null) {
       if (!threw)
-        timerEmitAfter(timerAsyncId);
+        emitAfter(timerAsyncId);
       if (!timer._repeat && async_hook_fields[kDestroy] > 0 &&
           !timer._destroyed) {
         emitDestroy(timerAsyncId);
@@ -467,7 +448,7 @@ exports.setTimeout = setTimeout;
 function createSingleTimeout(callback, after, args) {
   after *= 1; // coalesce to number or NaN
   if (!(after >= 1 && after <= TIMEOUT_MAX))
-    after = 1; // schedule on next tick, follows browser behaviour
+    after = 1; // schedule on next tick, follows browser behavior
 
   var timer = new Timeout(after, callback, args);
   if (process.domain)
@@ -556,7 +537,7 @@ exports.setInterval = function(callback, repeat, arg1, arg2, arg3) {
 function createRepeatTimeout(callback, repeat, args) {
   repeat *= 1; // coalesce to number or NaN
   if (!(repeat >= 1 && repeat <= TIMEOUT_MAX))
-    repeat = 1; // schedule on next tick, follows browser behaviour
+    repeat = 1; // schedule on next tick, follows browser behavior
 
   var timer = new Timeout(repeat, callback, args);
   timer._repeat = repeat;
@@ -756,16 +737,16 @@ function processImmediate() {
 // 4.7) what is in this smaller function.
 function tryOnImmediate(immediate, oldTail) {
   var threw = true;
-  timerEmitBefore(immediate[async_id_symbol], immediate[trigger_id_symbol]);
+  emitBefore(immediate[async_id_symbol], immediate[trigger_id_symbol]);
   try {
-    // make the actual call outside the try/catch to allow it to be optimized
+    // make the actual call outside the try/finally to allow it to be optimized
     runCallback(immediate);
     threw = false;
   } finally {
     // clearImmediate checks _callback === null for kDestroy hooks.
     immediate._callback = null;
     if (!threw)
-      timerEmitAfter(immediate[async_id_symbol]);
+      emitAfter(immediate[async_id_symbol]);
     if (async_hook_fields[kDestroy] > 0 && !immediate._destroyed) {
       emitDestroy(immediate[async_id_symbol]);
       immediate._destroyed = true;
