@@ -127,8 +127,8 @@ inline v8::Local<v8::String> Environment::AsyncHooks::provider_string(int idx) {
 
 inline void Environment::AsyncHooks::push_ids(double async_id,
                                               double trigger_id) {
-  CHECK_GE(async_id, 0);
-  CHECK_GE(trigger_id, 0);
+  CHECK_GE(async_id, -1);
+  CHECK_GE(trigger_id, -1);
 
   ids_stack_.push({ uid_fields_[kCurrentAsyncId],
                     uid_fields_[kCurrentTriggerId] });
@@ -166,6 +166,10 @@ inline bool Environment::AsyncHooks::pop_ids(double async_id) {
   return !ids_stack_.empty();
 }
 
+inline size_t Environment::AsyncHooks::stack_size() {
+  return ids_stack_.size();
+}
+
 inline void Environment::AsyncHooks::clear_id_stack() {
   while (!ids_stack_.empty())
     ids_stack_.pop();
@@ -176,13 +180,14 @@ inline void Environment::AsyncHooks::clear_id_stack() {
 inline Environment::AsyncHooks::InitScope::InitScope(
     Environment* env, double init_trigger_id)
         : env_(env),
-          uid_fields_(env->async_hooks()->uid_fields()) {
-  env->async_hooks()->push_ids(uid_fields_[AsyncHooks::kCurrentAsyncId],
+          uid_fields_ref_(env->async_hooks()->uid_fields()) {
+  CHECK_GE(init_trigger_id, -1);
+  env->async_hooks()->push_ids(uid_fields_ref_[AsyncHooks::kCurrentAsyncId],
                                init_trigger_id);
 }
 
 inline Environment::AsyncHooks::InitScope::~InitScope() {
-  env_->async_hooks()->pop_ids(uid_fields_[AsyncHooks::kCurrentAsyncId]);
+  env_->async_hooks()->pop_ids(uid_fields_ref_[AsyncHooks::kCurrentAsyncId]);
 }
 
 inline Environment::AsyncHooks::ExecScope::ExecScope(
@@ -190,6 +195,8 @@ inline Environment::AsyncHooks::ExecScope::ExecScope(
         : env_(env),
           async_id_(async_id),
           disposed_(false) {
+  CHECK_GE(async_id, -1);
+  CHECK_GE(trigger_id, -1);
   env->async_hooks()->push_ids(async_id, trigger_id);
 }
 
@@ -259,6 +266,9 @@ inline void Environment::TickInfo::set_index(uint32_t value) {
 
 inline void Environment::AssignToContext(v8::Local<v8::Context> context) {
   context->SetAlignedPointerInEmbedderData(kContextEmbedderDataIndex, this);
+#if HAVE_INSPECTOR
+  inspector_agent()->ContextCreated(context);
+#endif  // HAVE_INSPECTOR
 }
 
 inline Environment* Environment::GetCurrent(v8::Isolate* isolate) {
@@ -272,18 +282,16 @@ inline Environment* Environment::GetCurrent(v8::Local<v8::Context> context) {
 
 inline Environment* Environment::GetCurrent(
     const v8::FunctionCallbackInfo<v8::Value>& info) {
-  ASSERT(info.Data()->IsExternal());
+  CHECK(info.Data()->IsExternal());
   return static_cast<Environment*>(info.Data().As<v8::External>()->Value());
 }
 
 template <typename T>
 inline Environment* Environment::GetCurrent(
     const v8::PropertyCallbackInfo<T>& info) {
-  ASSERT(info.Data()->IsExternal());
-  // XXX(bnoordhuis) Work around a g++ 4.9.2 template type inferrer bug
-  // when the expression is written as info.Data().As<v8::External>().
-  v8::Local<v8::Value> data = info.Data();
-  return static_cast<Environment*>(data.As<v8::External>()->Value());
+  CHECK(info.Data()->IsExternal());
+  return static_cast<Environment*>(
+      info.Data().template As<v8::External>()->Value());
 }
 
 inline Environment::Environment(IsolateData* isolate_data,
@@ -292,8 +300,6 @@ inline Environment::Environment(IsolateData* isolate_data,
       isolate_data_(isolate_data),
       async_hooks_(context->GetIsolate()),
       timer_base_(uv_now(isolate_data->event_loop())),
-      cares_query_last_ok_(true),
-      cares_is_servers_default_(true),
       using_domains_(false),
       printed_error_(false),
       trace_sync_io_(false),
@@ -313,10 +319,19 @@ inline Environment::Environment(IsolateData* isolate_data,
   set_binding_cache_object(v8::Object::New(isolate()));
   set_module_load_list_array(v8::Array::New(isolate()));
 
-  RB_INIT(&cares_task_list_);
   AssignToContext(context);
 
   destroy_ids_list_.reserve(512);
+  performance_state_ = Calloc<performance::performance_state>(1);
+  performance_state_->milestones[
+      performance::NODE_PERFORMANCE_MILESTONE_ENVIRONMENT] =
+          PERFORMANCE_NOW();
+  performance_state_->milestones[
+    performance::NODE_PERFORMANCE_MILESTONE_NODE_START] =
+        performance::performance_node_start;
+  performance_state_->milestones[
+    performance::NODE_PERFORMANCE_MILESTONE_V8_START] =
+        performance::performance_v8_start;
 }
 
 inline Environment::~Environment() {
@@ -331,6 +346,8 @@ inline Environment::~Environment() {
   delete[] heap_statistics_buffer_;
   delete[] heap_space_statistics_buffer_;
   delete[] http_parser_buffer_;
+  free(http2_state_buffer_);
+  free(performance_state_);
 }
 
 inline v8::Isolate* Environment::isolate() const {
@@ -471,7 +488,6 @@ inline void Environment::set_heap_space_statistics_buffer(double* pointer) {
   heap_space_statistics_buffer_ = pointer;
 }
 
-
 inline char* Environment::http_parser_buffer() const {
   return http_parser_buffer_;
 }
@@ -481,6 +497,15 @@ inline void Environment::set_http_parser_buffer(char* buffer) {
   http_parser_buffer_ = buffer;
 }
 
+inline http2::http2_state* Environment::http2_state_buffer() const {
+  return http2_state_buffer_;
+}
+
+inline void Environment::set_http2_state_buffer(http2::http2_state* buffer) {
+  CHECK_EQ(http2_state_buffer_, nullptr);  // Should be set only once.
+  http2_state_buffer_ = buffer;
+}
+
 inline double* Environment::fs_stats_field_array() const {
   return fs_stats_field_array_;
 }
@@ -490,41 +515,39 @@ inline void Environment::set_fs_stats_field_array(double* fields) {
   fs_stats_field_array_ = fields;
 }
 
-inline Environment* Environment::from_cares_timer_handle(uv_timer_t* handle) {
-  return ContainerOf(&Environment::cares_timer_handle_, handle);
+inline performance::performance_state* Environment::performance_state() {
+  return performance_state_;
 }
 
-inline uv_timer_t* Environment::cares_timer_handle() {
-  return &cares_timer_handle_;
+inline std::map<std::string, uint64_t>* Environment::performance_marks() {
+  return &performance_marks_;
 }
 
-inline ares_channel Environment::cares_channel() {
-  return cares_channel_;
+inline Environment* Environment::from_performance_check_handle(
+    uv_check_t* handle) {
+  return ContainerOf(&Environment::performance_check_handle_, handle);
 }
 
-// Only used in the call to ares_init_options().
-inline ares_channel* Environment::cares_channel_ptr() {
-  return &cares_channel_;
+inline Environment* Environment::from_performance_idle_handle(
+    uv_idle_t* handle) {
+  return ContainerOf(&Environment::performance_idle_handle_, handle);
 }
 
-inline bool Environment::cares_query_last_ok() {
-  return cares_query_last_ok_;
+inline Environment* Environment::from_performance_prepare_handle(
+    uv_prepare_t* handle) {
+  return ContainerOf(&Environment::performance_prepare_handle_, handle);
 }
 
-inline void Environment::set_cares_query_last_ok(bool ok) {
-  cares_query_last_ok_ = ok;
+inline uv_check_t* Environment::performance_check_handle() {
+  return &performance_check_handle_;
 }
 
-inline bool Environment::cares_is_servers_default() {
-  return cares_is_servers_default_;
+inline uv_idle_t* Environment::performance_idle_handle() {
+  return &performance_idle_handle_;
 }
 
-inline void Environment::set_cares_is_servers_default(bool is_default) {
-  cares_is_servers_default_ = is_default;
-}
-
-inline node_ares_task_list* Environment::cares_task_list() {
-  return &cares_task_list_;
+inline uv_prepare_t* Environment::performance_prepare_handle() {
+  return &performance_prepare_handle_;
 }
 
 inline IsolateData* Environment::isolate_data() const {
