@@ -23,8 +23,8 @@
 #define SRC_NODE_VERSION_H_
 
 #define NODE_MAJOR_VERSION 8
-#define NODE_MINOR_VERSION 1
-#define NODE_PATCH_VERSION 3
+#define NODE_MINOR_VERSION 5
+#define NODE_PATCH_VERSION 0
 
 #define NODE_VERSION_IS_RELEASE 1
 
@@ -33,6 +33,10 @@
 #define NODE_STRINGIFY_HELPER(n) #n
 #endif
 
+#ifndef NODE_RELEASE
+#define NODE_RELEASE "node"
+#endif
+
 #ifndef NODE_TAG
 # if NODE_VERSION_IS_RELEASE
 #  define NODE_TAG ""
