@@ -21,6 +21,10 @@
 
 'use strict';
 const common = require('../common');
+// Skip test in FreeBSD jails.
+if (common.inFreeBSDJail)
+  common.skip('In a FreeBSD jail');
+
 const assert = require('assert');
 const dgram = require('dgram');
 const fork = require('child_process').fork;
@@ -37,12 +41,6 @@ const listeners = 3;
 let listening, sendSocket, done, timer, dead;
 
 
-// Skip test in FreeBSD jails.
-if (common.inFreeBSDJail) {
-  common.skip('In a FreeBSD jail');
-  return;
-}
-
 function launchChildProcess() {
   const worker = fork(__filename, ['child']);
   workers[worker.pid] = worker;
