@@ -7,8 +7,9 @@ const common = require('../common');
 
 const assert = require('assert');
 const fork = require('child_process').fork;
+const fixtures = require('../common/fixtures');
 
-const childScript = `${common.fixturesDir}/child-process-spawn-node`;
+const childScript = fixtures.path('child-process-spawn-node');
 const errorRegexp = /^TypeError: Incorrect value of stdio option:/;
 const malFormedOpts = {stdio: '33'};
 const payload = {hello: 'world'};
