@@ -29,7 +29,7 @@ const norepeat = [
 ];
 
 const server = http.createServer(function(req, res) {
-  var num = req.headers['x-num'];
+  const num = req.headers['x-num'];
   if (num === '1') {
     for (const name of norepeat) {
       res.setHeader(name, ['A', 'B']);
@@ -47,8 +47,8 @@ const server = http.createServer(function(req, res) {
 });
 
 server.listen(0, common.mustCall(function() {
-  var count = 0;
-  for (let n = 1; n <= 2 ; n++) {
+  let count = 0;
+  for (let n = 1; n <= 2; n++) {
     // this runs twice, the first time, the server will use
     // setHeader, the second time it uses writeHead. The
     // result on the client side should be the same in
@@ -60,9 +60,9 @@ server.listen(0, common.mustCall(function() {
       common.mustCall(function(res) {
         if (++count === 2) server.close();
         for (const name of norepeat) {
-          assert.equal(res.headers[name], 'A');
+          assert.strictEqual(res.headers[name], 'A');
         }
-        assert.equal(res.headers['x-a'], 'A, B');
+        assert.strictEqual(res.headers['x-a'], 'A, B');
       })
     );
   }
