@@ -7,11 +7,11 @@ if (!common.hasIntl) {
 }
 
 const URL = require('url').URL;
-const path = require('path');
 const assert = require('assert');
+const fixtures = require('../common/fixtures');
 
 // Tests below are not from WPT.
-const tests = require(path.join(common.fixturesDir, 'url-tests'));
+const tests = require(fixtures.path('url-tests'));
 const failureTests = tests.filter((test) => test.failure).concat([
   { input: '' },
   { input: 'test' },
@@ -26,7 +26,7 @@ const failureTests = tests.filter((test) => test.failure).concat([
 ]);
 
 const expectedError = common.expectsError(
-    { code: 'ERR_INVALID_URL', type: TypeError });
+  { code: 'ERR_INVALID_URL', type: TypeError }, 102);
 
 for (const test of failureTests) {
   assert.throws(
@@ -44,8 +44,8 @@ for (const test of failureTests) {
     });
 }
 
-const additional_tests = require(
-  path.join(common.fixturesDir, 'url-tests-additional.js'));
+const additional_tests =
+  require(fixtures.path('url-tests-additional.js'));
 
 for (const test of additional_tests) {
   const url = new URL(test.url);
