@@ -27,23 +27,19 @@
 // TLS server causes the child process to exit cleanly before having sent
 // the entire buffer.
 const common = require('../common');
+if (!common.hasCrypto)
+  common.skip('missing crypto');
+
 const assert = require('assert');
 const spawn = require('child_process').spawn;
-
-if (!common.hasCrypto) {
-  common.skip('missing crypto');
-  return;
-}
 const https = require('https');
-
-const fs = require('fs');
+const fixtures = require('../common/fixtures');
 
 const bytesExpected = 1024 * 1024 * 32;
 
 let started = false;
 
-const childScript = require('path').join(common.fixturesDir,
-                                         'GH-892-request.js');
+const childScript = fixtures.path('GH-892-request.js');
 
 function makeRequest() {
   if (started) return;
@@ -81,8 +77,8 @@ function makeRequest() {
 
 
 const serverOptions = {
-  key: fs.readFileSync(`${common.fixturesDir}/keys/agent1-key.pem`),
-  cert: fs.readFileSync(`${common.fixturesDir}/keys/agent1-cert.pem`)
+  key: fixtures.readKey('agent1-key.pem'),
+  cert: fixtures.readKey('agent1-cert.pem')
 };
 
 let uploadCount = 0;
