@@ -20,24 +20,18 @@
 // USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 'use strict';
-//
 
 const common = require('../common');
 
-if (!common.opensslCli) {
+if (!common.opensslCli)
   common.skip('node compiled without OpenSSL CLI.');
-  return;
-}
 
-if (!common.hasCrypto) {
+if (!common.hasCrypto)
   common.skip('missing crypto');
-  return;
-}
 
-const join = require('path').join;
 const net = require('net');
 const assert = require('assert');
-const fs = require('fs');
+const fixtures = require('../common/fixtures');
 const tls = require('tls');
 const spawn = require('child_process').spawn;
 
@@ -61,11 +55,8 @@ function test2() {
 }
 
 function test(keyfn, certfn, check, next) {
-  keyfn = join(common.fixturesDir, keyfn);
-  const key = fs.readFileSync(keyfn).toString();
-
-  certfn = join(common.fixturesDir, certfn);
-  const cert = fs.readFileSync(certfn).toString();
+  const key = fixtures.readSync(keyfn).toString();
+  const cert = fixtures.readSync(certfn).toString();
 
   const server = spawn(common.opensslCli, ['s_server',
                                            '-accept', common.PORT,
@@ -84,7 +75,7 @@ function test(keyfn, certfn, check, next) {
     console.error(state);
     switch (state) {
       case 'WAIT-ACCEPT':
-        if (/ACCEPT/g.test(serverStdoutBuffer)) {
+        if (/ACCEPT/.test(serverStdoutBuffer)) {
           // Give s_server half a second to start up.
           setTimeout(startClient, 500);
           state = 'WAIT-HELLO';
@@ -92,7 +83,7 @@ function test(keyfn, certfn, check, next) {
         break;
 
       case 'WAIT-HELLO':
-        if (/hello/g.test(serverStdoutBuffer)) {
+        if (/hello/.test(serverStdoutBuffer)) {
 
           // End the current SSL connection and exit.
           // See s_server(1ssl).
