@@ -15,8 +15,14 @@ const path = require('path');
 
 const runjs = path.join(__dirname, '..', '..', 'benchmark', 'run.js');
 
-const child = fork(runjs, ['--set', 'dur=0', 'net'],
-                   {env: {NODEJS_BENCHMARK_ZERO_ALLOWED: 1}});
+const env = Object.assign({}, process.env,
+                          { NODEJS_BENCHMARK_ZERO_ALLOWED: 1 });
+const child = fork(runjs,
+                   ['--set', 'dur=0',
+                    '--set', 'len=1024',
+                    '--set', 'type=buf',
+                    'net'],
+                   { env });
 child.on('exit', (code, signal) => {
   assert.strictEqual(code, 0);
   assert.strictEqual(signal, null);
